-- DCS - Iron Bombing Range Script
-- Version 1.1

-- Based on original by
-- By Ciribob - https://github.com/ciribob/DCS-SimpleRangeScript
-- Inspired by Original Script by SNAFU http://forums.eagle.ru/showthread.php?t=109174
-- Changelog
-- Removed reporting code
-- Removed strafing code
-- Updated to account for changes to DCS on Jan 2021 that changed weapon naming conventions
-- added direction of hit from target
-- Requires MIST 4.0.57 or newer!


range  = {} 
trigger.action.outText( 'Iron Bombing Range Script Loaded - V1.1', 10 )

-- Zone Names
range.bombingMinAlt = 0
range.bombingTargets = {

    "target1",
    "target2",
    "target3",
    "target4",
    "target5",
    "target6",
    "target7",
    "target8",
    "target9",
    "target10",
    "target11",
    "target12",
    "target13",
    "target14",
    "target15",

}


-- Handles all world events
range.eventHandler = {}
function range.eventHandler:onEvent(_eventDCS)

    if _eventDCS == nil or _eventDCS.initiator == nil then
        return true
    end


    local status, err = pcall(function(_event)
    
        if _event.id == world.event.S_EVENT_SHOT then

            local _weapon = _event.weapon:getTypeName()
 --           local _weaponStrArray = range.split(_weapon,"%.")
 --           local _weaponName = _weaponStrArray[#_weaponStrArray]
              -- Note It seems that DCS used to append "weapons.bombs" or "Weapons.nurs" to the weapon names
              -- but as of Jan 2021, it no longer does.  For now, it will be necessary to go through the list of
              -- bombs and rockets one at a time

            env.info("Dropping ".._weapon)
   
            trigger.action.outText( 'Bomb Drop '.._event.weapon:getTypeName(), 10 )
            if _weapon == "Mk_83" -- Mk_83
              or _weapon == "Mk_82" -- Mk_82
              or _weapon == "MK_82SNAKEYE"
              or _weapon == "ROCKEYE"-- Rockeye
              or _weapon == "FAB_250"-- FAB 250 Russian bomb
              or _weapon == "C_13"-- 130mm Russian rocket
              or _weapon == "C_24"-- S24 Russian Rocket 
              
				    and _event.initiator:getPosition().p.y > range.bombingMinAlt
            then
                local _ordnance =  _event.weapon
                env.info("Tracking ".._weapon.." - ".._ordnance:getName())
                -- trigger.action.outText( "Tracking ".._weapon, 10 )
                local _lastBombPos = {x=0,y=0,z=0}

                local _unitName = _event.initiator:getName()
                local trackBomb = function(_previousPos)

                    local _unit = Unit.getByName(_unitName)
                    --    env.info("Checking...")
                    if _unit ~= nil and _unit:getPlayerName() ~= nil then

                        -- when the pcall returns a failure the weapon has hit
                        local _status,_bombPos =  pcall(function()
                            -- env.info("protected")
                            return _ordnance:getPoint()
                        end)

                        if  _status then
                            --ok! still in the air
                            _lastBombPos = {x = _bombPos.x, y = _bombPos.y, z= _bombPos.z }

                            return timer.getTime() + 0.005 -- check again !
                        else
                            --hit
                            -- get closest target to last position
                            local _closetTarget = nil
                            local _distance = nil
                            local _direction = nil

                            for _,_targetZone in pairs(range.bombingTargets) do

                                local _temp = range.getDistance(_targetZone.point, _lastBombPos)

                                if _distance == nil or _temp < _distance then
                                    _distance = _temp
                                    _closetTarget = _targetZone
                                    _direction = range.getDirection(_targetZone.point, _lastBombPos)
                                end
                            end

                            env.info(_distance.." from ".._closetTarget.name.." @".._direction)

                            if _distance < 1000 then
                                 local _DistResult
                                 _DistResult = string.format(" %i m from ", _distance)
                                 _DirResult = string.format(" bearing %i degrees", _direction)
                                 trigger.action.outText(_weapon.._DistResult.._closetTarget.name.._DirResult, 10,10,false)

                            end
                        end

                    end

                    return
                end

                timer.scheduleFunction(trackBomb, nil, timer.getTime() + 1)
            end
        end


        return true

    end, _eventDCS)

    if (not status) then
        env.error(string.format("Error while handling event %s", err),false)
    end
end


function range.getGroupId(_unit)

    local _unitDB =  mist.DBs.unitsById[tonumber(_unit:getID())]
    if _unitDB ~= nil and _unitDB.groupId then
        return _unitDB.groupId
    end

    return nil
end

--get distance in meters assuming a Flat world
function range.getDistance(_point1, _point2)

    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z

    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone

    return math.sqrt(xDiff * xDiff + yDiff * yDiff)
end
function range.getDirection(_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
    tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
    tgtBearing = 360 + tgtAngle
    end
    
    
 env.info("xDiff "..xDiff.." yDiff "..yDiff)    
    return tgtBearing
end
--http://stackoverflow.com/questions/1426954/split-string-in-lua
function range.split(str, sep)
    local result = {}
    local regex = ("([^%s]+)"):format(sep)
    for each in str:gmatch(regex) do
        table.insert(result, each)
    end
    return result
end

--init

local _tempTargets = range.bombingTargets

range.bombingTargets = {}

for _,_targetZone in pairs(_tempTargets) do

    local _triggerZone = trigger.misc.getZone(_targetZone)

    if _triggerZone then
        table.insert(range.bombingTargets,{name=_targetZone,point=_triggerZone.point})
        env.info("Done for: ".._targetZone)
    else
        env.info("Failed for: ".._targetZone)

    end

end


world.addEventHandler(range.eventHandler)

